@@ -70,7 +70,12 @@ def main(args=None, steam_path=None, steam_root=None):
             "Runtime, valid path = custom Steam Runtime path, "
             "empty = enable automatically (default)\n"
             "PROTONTRICKS_GUI: GUI provider to use, accepts either 'yad' "
-            "or 'zenity'"
+            "or 'zenity'\n"
+            "\n"
+            "Environment variables set automatically by Protontricks:\n"
+            "STEAM_APP_PATH: path to the current game's installation directory\n"
+            "STEAM_APPID: app ID of the current game\n"
+            "PROTON_PATH: path to the currently used Proton installation"
         ),
         formatter_class=argparse.RawTextHelpFormatter
     )
@@ -99,8 +104,7 @@ def main(args=None, steam_path=None, steam_root=None):
     parser.add_argument(
         "-c", "--command", type=str, dest="command",
         required=False,
-        help="Run a command in the game's installation directory with "
-             "Wine-related environment variables set. "
+        help="Run a command with Wine-related environment variables set. "
              "The command is passed to the shell as-is without being escaped.")
     parser.add_argument(
         "--gui", action="store_true",
@@ -131,6 +135,16 @@ def main(args=None, steam_path=None, steam_root=None):
             "command startup time."
         )
     )
+    parser.add_argument(
+        "--cwd-app",
+        dest="cwd_app",
+        default=False,
+        action="store_true",
+        help=(
+            "Set the working directory of launched command to the Steam app's "
+            "installation directory."
+        )
+    )
     parser.set_defaults(background_wineserver=False)
 
     parser.add_argument("appid", type=int, nargs="?", default=None)
@@ -142,6 +156,10 @@ def main(args=None, steam_path=None, steam_root=None):
 
     args = parser.parse_args(args)
 
+    if len(sys.argv) < 2:
+        # No arguments were provided, default to GUI
+        args.gui = True
+
     # 'cli_error_handler' relies on this to know whether to use error dialog or
     # not
     main.no_term = args.no_term
@@ -273,6 +291,8 @@ def main(args=None, steam_path=None, steam_root=None):
                 "Protontricks GUI."
             )
 
+        cwd = str(steam_app.install_path) if args.cwd_app else None
+
         # 6. Find Proton version of selected app
         proton_app = find_proton_app(
             steam_path=steam_path, steam_apps=steam_apps, appid=steam_app.appid
@@ -287,6 +307,7 @@ def main(args=None, steam_path=None, steam_root=None):
                 "installation?"
             )
 
+
         run_command(
             winetricks_path=winetricks_path,
             proton_app=proton_app,
@@ -295,7 +316,8 @@ def main(args=None, steam_path=None, steam_root=None):
             legacy_steam_runtime_path=legacy_steam_runtime_path,
             command=[str(winetricks_path), "--gui"],
             use_bwrap=use_bwrap,
-            start_wineserver=start_background_wineserver
+            start_wineserver=start_background_wineserver,
+            cwd=cwd
         )
 
         return
@@ -366,6 +388,8 @@ def main(args=None, steam_path=None, steam_root=None):
             "$ protontricks -s <GAME NAME>"
         )
 
+    cwd = str(steam_app.install_path) if args.cwd_app else None
+
     if args.winetricks_command:
         returncode = run_command(
             winetricks_path=winetricks_path,
@@ -375,7 +399,8 @@ def main(args=None, steam_path=None, steam_root=None):
             legacy_steam_runtime_path=legacy_steam_runtime_path,
             use_bwrap=use_bwrap,
             start_wineserver=start_background_wineserver,
-            command=[str(winetricks_path)] + args.winetricks_command
+            command=[str(winetricks_path)] + args.winetricks_command,
+            cwd=cwd
         )
     elif args.command:
         returncode = run_command(
@@ -389,8 +414,8 @@ def main(args=None, steam_path=None, steam_root=None):
             start_wineserver=start_background_wineserver,
             # Pass the command directly into the shell *without*
             # escaping it
-            cwd=str(steam_app.install_path),
-            shell=True
+            shell=True,
+            cwd=cwd,
         )
 
     logger.info("Command returned %d", returncode)
