@@ -15,6 +15,7 @@ from PIL import Image
 from .config import get_config
 from .flatpak import get_inaccessible_paths
 from .util import get_cache_dir
+from .steam import SNAP_STEAM_DIRS
 
 APP_ICON_SIZE = (32, 32)
 
@@ -54,7 +55,7 @@ def get_gui_provider():
         ) from exc
 
 
-def _get_appid2icon(steam_apps, steam_path):
+def _get_appid2icon(steam_apps):
     """
     Get icons for Steam apps to show in the app selection dialog.
     Return a {appid: icon_path} dict.
@@ -65,9 +66,6 @@ def _get_appid2icon(steam_apps, steam_path):
         )
     )
 
-    steam_icon_dir = steam_path / "appcache" / "librarycache"
-    existing_names = [path.name for path in steam_icon_dir.glob("*")]
-
     protontricks_icon_dir = get_cache_dir() / "app_icons"
     protontricks_icon_dir.mkdir(exist_ok=True)
 
@@ -77,39 +75,43 @@ def _get_appid2icon(steam_apps, steam_path):
         # Use library icon for Steam apps, fallback to placeholder icon
         # for non-Steam shortcuts and missing icons
         icon_cache_path = protontricks_icon_dir / f"{app.appid}.jpg"
-        original_icon_path = steam_icon_dir / f"{app.appid}_icon.jpg"
 
         # What path to actually use for the app selector icon
         final_icon_path = placeholder_path
 
-        icon_exists = f"{app.appid}_icon.jpg" in existing_names
-        resize_icon = False
+        if app.icon_path:
+            # Resize icons that have a non-standard size to ensure they can be
+            # displayed consistently in the app selector
+            try:
+                with Image.open(app.icon_path) as img:
+                    # Icon exists, so use the current icon instead of the
+                    # default placeholder.
+                    final_icon_path = app.icon_path
 
-        # Resize icons that have a non-standard size to ensure they can be
-        # displayed consistently in the app selector
-        if icon_exists:
-            final_icon_path = original_icon_path
+                    resize_icon = img.size != APP_ICON_SIZE
 
-            with Image.open(original_icon_path) as img:
-                resize_icon = img.size != APP_ICON_SIZE
-
-                # Resize icons that have a non-standard size to ensure they can
-                # be displayed consistently in the app selector
-                if resize_icon:
-                    logger.info(
-                        "App icon %s has unusual size, resizing",
-                        original_icon_path
-                    )
-                    try:
+                    # Resize icons that have a non-standard size to ensure they can
+                    # be displayed consistently in the app selector
+                    if resize_icon:
+                        logger.info(
+                            "App icon %s has unusual size, resizing",
+                            app.icon_path
+                        )
                         resized_img = img.resize(APP_ICON_SIZE).convert("RGB")
                         resized_img.save(icon_cache_path)
                         final_icon_path = icon_cache_path
-                    except Exception:
-                        logger.warning(
-                            "Could not resize %s, ignoring",
-                            original_icon_path,
-                            exc_info=True
-                        )
+            except FileNotFoundError:
+                # Icon does not exist, the placeholder will be used
+                pass
+            except Exception:
+                # Multitude of reasons can cause image parsing or resizing
+                # to fail. Instead of trying to catch everything, log the error
+                # and move on.
+                logger.warning(
+                    "Could not resize %s, ignoring",
+                    app.icon_path,
+                    exc_info=True
+                )
 
         appid2icon[app.appid] = final_icon_path
 
@@ -250,7 +252,17 @@ def select_steam_installation(steam_installations):
                 "/com.valvesoftware.Steam/.local/share/Steam"
             )
         )
-        install_type = "Flatpak" if is_flatpak else "Native"
+        is_snap = any(
+            str(steam_path).endswith(snap_dir)
+            for snap_dir in SNAP_STEAM_DIRS
+        )
+
+        if is_flatpak:
+            install_type = "Flatpak"
+        elif is_snap:
+            install_type = "Snap"
+        else:
+            install_type = "Native"
 
         cmd_input.append(f"{i+1}: {install_type} - {steam_path}")
 
@@ -270,7 +282,9 @@ def select_steam_installation(steam_installations):
             # No installation was selected
             choice = b""
         else:
-            raise RuntimeError(f"{gui_provider} returned an error")
+            raise RuntimeError(
+                f"{gui_provider} returned an error. Stderr: {exc.stderr}"
+            )
 
     if choice in (b"", b" \n"):
         return None, None
@@ -322,7 +336,7 @@ def select_steam_app_with_gui(steam_apps, steam_path, title=None):
         args = _get_yad_args()
 
         # YAD implementation has icons for app selection
-        appid2icon = _get_appid2icon(steam_apps, steam_path=steam_path)
+        appid2icon = _get_appid2icon(steam_apps)
 
         cmd_input = [
             [
@@ -368,7 +382,9 @@ def select_steam_app_with_gui(steam_apps, steam_path, title=None):
             # No game was selected
             choice = b""
         else:
-            raise RuntimeError(f"{gui_provider} returned an error")
+            raise RuntimeError(
+                f"{gui_provider} returned an error. Stderr: {exc.stderr}"
+            )
 
     if choice in (b"", b" \n"):
         print("No game was selected. Quitting...")
@@ -411,11 +427,17 @@ def prompt_filesystem_access(paths, show_dialog=False):
     inaccessible_paths = get_inaccessible_paths(paths)
     inaccessible_paths = set(map(str, inaccessible_paths))
 
+    logger.debug(
+        "Following inaccessible paths were found: %s", inaccessible_paths
+    )
+
     # Check what paths the user has ignored previously
     ignored_paths = set(
         json.loads(config.get("Dialog", "DismissedPaths", "[]"))
     )
 
+    logger.debug("Following paths have been ignored: %s", ignored_paths)
+
     # Remaining paths that are inaccessible and that haven't been dismissed
     # by the user
     remaining_paths = inaccessible_paths - ignored_paths
