@@ -1,4 +1,3 @@
-import functools
 import logging
 import os
 import string
@@ -9,8 +8,8 @@ from pathlib import Path
 
 import vdf
 
-from .flatpak import is_flatpak_sandbox
-from .util import lower_dict, is_steam_deck
+from ._vdf import binary_loads as vendored_binary_loads
+from .util import is_steam_deck, lower_dict
 
 __all__ = (
     "COMMON_STEAM_DIRS", "SteamApp", "find_steam_installations",
@@ -29,6 +28,13 @@ COMMON_STEAM_DIRS = [
     ".local/share/Steam",
 ]
 
+SNAP_STEAM_DIRS = [
+    "snap/steam/common/.local/share/Steam",
+
+    # Location if 'experimental.hidden-snap-folder' Snap feature flag is set
+    ".snap/data/steam/common/.local/share/Steam",
+]
+
 OS_RELEASE_PATHS = [
     "/run/host/os-release",  # The host file if we're inside a Flatpak sandbox
     "/etc/os-release"
@@ -48,19 +54,21 @@ class SteamApp(object):
     Proton prefix)
     """
     __slots__ = (
-        "appid", "name", "prefix_path", "install_path", "required_tool_appid",
-        "required_tool_app"
+        "appid", "name", "prefix_path", "install_path", "icon_path",
+        "required_tool_appid", "required_tool_app"
     )
 
     def __init__(
-            self, name, install_path, prefix_path=None, appid=None,
-            required_tool_appid=None):
+            self, name, install_path, icon_path=None, prefix_path=None,
+            appid=None, required_tool_appid=None):
         """
         :appid: App's appid
         :name: The app's human-readable name
         :prefix_path: Absolute path to where the app's Wine prefix *might*
                       exist.
         :app_path: Absolute path to app's installation directory
+        :icon_path: Absolute path to app's icon, whether Steam's own or custom
+                    one configured by user
         :required_tool_appid: App ID required to run this application.
                               Usually corresponds to a Steam Runtime for
                               Proton installations.
@@ -76,6 +84,11 @@ class SteamApp(object):
         else:
             self.prefix_path = None
 
+        if icon_path:
+            self.icon_path = Path(icon_path)
+        else:
+            self.icon_path = None
+
         self.install_path = Path(install_path)
 
         # Reference to another SteamApp will be added later if necessary,
@@ -181,11 +194,17 @@ class SteamApp(object):
             return None
 
     @classmethod
-    def from_appmanifest(cls, path, steam_lib_paths):
+    def from_appmanifest(cls, path, steam_lib_paths, steam_path=None):
         """
         Parse appmanifest_X.acf file containing Steam app installation metadata
         and return a SteamApp object
+
+        If 'steam_path' is provided, icon path is also populated
         """
+        logger.debug(
+            "Creating SteamApp from manifest file in %s", path
+        )
+
         try:
             content = path.read_text(encoding="utf-8")
         except UnicodeDecodeError:
@@ -240,6 +259,13 @@ class SteamApp(object):
 
         install_path = Path(path).parent / "common" / app_state["installdir"]
 
+        icon_path = None
+
+        # If Steam path was provided, also populate the icon path
+        if steam_path:
+            icon_path = \
+                steam_path / "appcache" / "librarycache" / f"{appid}_icon.jpg"
+
         # Check if the app requires another app. This is the case with
         # newer versions of Proton, which use Steam Runtimes installed as
         # normal Steam apps
@@ -255,7 +281,8 @@ class SteamApp(object):
 
         return cls(
             appid=appid, name=name, prefix_path=prefix_path,
-            install_path=install_path, required_tool_appid=required_tool_appid
+            install_path=install_path, icon_path=icon_path,
+            required_tool_appid=required_tool_appid
         )
 
 
@@ -369,14 +396,19 @@ def find_steam_installations():
 
             candidates[(str(steam_path), str(steam_root_))] = True
 
-    # Check for Flatpak Steam separately and ensure we don't mix steam_root
-    # and steam_path from Flatpak and non-Flatpak installations of Steam.
+    # Check for Flatpak and Snap Steam separately and ensure we don't mix steam_root
+    # and steam_path from Flatpak/Snap and native installations of Steam.
     steam_path = \
         Path.home() / ".var/app/com.valvesoftware.Steam/data/Steam"
     steam_path = steam_path.resolve()
     if has_steamapps_dir(steam_path):
         candidates[(str(steam_path), str(steam_path))] = True
 
+    for steam_dir in SNAP_STEAM_DIRS:
+        steam_path = (Path.home() / steam_dir).resolve()
+        if has_steamapps_dir(steam_path):
+            candidates[(str(steam_path), str(steam_path))] = True
+
     for steam_path, _ in candidates.keys():
         logger.info("Found Steam directory at %s", steam_path)
 
@@ -451,8 +483,8 @@ def find_legacy_steam_runtime_path(steam_root):
 
 
 APPINFO_STRUCT_HEADER = "<4sL"
-APPINFO_V27_STRUCT_SECTION = "<LLLLQ20sL"
 APPINFO_V28_STRUCT_SECTION = "<LLLLQ20sL20s"
+APPINFO_V29_STRUCT_SECTION = "<LLLLQ20sL20s"
 
 
 def iter_appinfo_sections(path):
@@ -460,34 +492,6 @@ def iter_appinfo_sections(path):
     Parse an appinfo.vdf file and iterate through all the binary VDF objects
     inside it
     """
-    def _iter_v27_appinfo(data, start):
-        """
-        Parse and iterate appinfo.vdf version 27
-        """
-        i = start
-        section_size = struct.calcsize(APPINFO_V27_STRUCT_SECTION)
-
-        while True:
-            # We don't need any of the fields besides 'entry_size',
-            # which is used to determine the length of the variable-length VDF
-            # field.
-            # Still, here they are for posterity's sake.
-            (appid, entry_size, infostate, last_updated, access_token,
-             sha_hash, change_number) = struct.unpack(
-                APPINFO_V27_STRUCT_SECTION, data[i:i+section_size])
-            vdf_section_size = entry_size - (section_size - 8)
-
-            i += section_size
-
-            vdf_d = vdf.binary_loads(data[i:i+vdf_section_size])
-            vdf_d = lower_dict(vdf_d)
-            yield vdf_d
-
-            i += vdf_section_size
-
-            if i == len(data) - 4:
-                return
-
     def _iter_v28_appinfo(data, start):
         """
         Parse and iterate appinfo.vdf version 28.
@@ -516,6 +520,71 @@ def iter_appinfo_sections(path):
             if i == len(data) - 4:
                 return
 
+    def _iter_v29_appinfo(data, start):
+        """
+        Parse and iterate appinfo.vdf version 29.
+        """
+        i = start
+
+        # The header contains the offset to the key table
+        key_table_offset = struct.unpack("<q", data[i:i+8])[0]
+        key_table = []
+
+        key_count = struct.unpack(
+            "<i", data[key_table_offset:key_table_offset+4]
+        )[0]
+
+        table_i = key_table_offset + 4
+        for _ in range(0, key_count):
+            key = bytearray()
+            while True:
+                key.append(data[table_i])
+                table_i += 1
+
+                if key[-1] == 0:
+                    key_table.append(
+                        key[0:-1].decode("utf-8", errors="replace")
+                    )
+                    break
+
+        i += 8
+
+        section_size = struct.calcsize(APPINFO_V29_STRUCT_SECTION)
+        while True:
+            # We don't need any of the fields besides 'entry_size',
+            # which is used to determine the length of the variable-length VDF
+            # field.
+            # Still, here they are for posterity's sake.
+            (appid, entry_size, infostate, last_updated, access_token,
+             sha_hash, change_number, vdf_sha_hash) = struct.unpack(
+                APPINFO_V29_STRUCT_SECTION, data[i:i+section_size])
+            vdf_section_size = entry_size - (section_size - 8)
+
+            i += section_size
+
+            try:
+                vdf_d = vdf.binary_loads(
+                    data[i:i+vdf_section_size], key_table=key_table
+                )
+            except TypeError:
+                # System 'vdf' is too old and does not support 'key_table',
+                # use the bundled one instead. This is cursed, but it's
+                # so far the only reasonable option without a proper maintained
+                # release on PyPI.
+                vdf_d = vendored_binary_loads(
+                    data[i:i+vdf_section_size], key_table=key_table
+                )
+
+            vdf_d = lower_dict(vdf_d)
+            yield vdf_d
+
+            i += vdf_section_size
+
+            if i == key_table_offset - 4:
+                return
+
+    logger.debug("Loading appinfo.vdf in %s", path)
+
     # appinfo.vdf is not actually a (binary) VDF file, but a binary file
     # containing multiple binary VDF sections.
     # File structure based on comment from vdf developer:
@@ -531,13 +600,18 @@ def iter_appinfo_sections(path):
 
     i += header_size
 
-    if magic == b"'DV\x07":
-        # TODO: Remove this once V28 is used in stable version of Steam
-        yield from _iter_v27_appinfo(data, i)
-    elif magic == b'(DV\x07':
+    logger.debug("appinfo.vdf has magic number %s", magic)
+
+    if magic == b'(DV\x07':
         yield from _iter_v28_appinfo(data, i)
+    elif magic == b')DV\x07':
+        yield from _iter_v29_appinfo(data, i)
     else:
-        raise SyntaxError("Invalid file magic number")
+        raise SyntaxError(
+            "Invalid file magic number. The appinfo.vdf version might not be "
+            "supported by the current version of Protontricks - please "
+            "check for updates."
+        )
 
 
 def get_appinfo_sections(path):
@@ -565,6 +639,8 @@ def get_tool_appid(compat_tool_name, steam_play_manifest):
         if "aliases" in entry:
             aliases += entry["aliases"].split(",")
 
+        logger.debug("%s has compat tool aliases %s", default_name, aliases)
+
         if compat_tool_name in aliases:
             return entry["appid"]
 
@@ -596,13 +672,18 @@ def find_steam_compat_tool_app(steam_path, steam_apps, appid=None):
         except StopIteration:
             return None
 
+    logger.debug("Finding Steam compat tool name for appid %s", appid)
+
     # Determine which Proton version to use (either globally or for
     # a specific app if `appid` was provided) by checking
     # `<steam_dir>/config/config.vdf` and `<steam_dir>/appcache/appinfo.vdf`.
     # Multiple configuration values might be found. In such case, select the
     # one with the highest priority.
     config_vdf_path = steam_path / "config" / "config.vdf"
-    content = config_vdf_path.read_text()
+    # config.vdf might contain invalid UTF-8 characters in the
+    # `SDL_GamepadBind` field. We don't use that in any way, so we can deal
+    # with the invalid characters by just ignoring them.
+    content = config_vdf_path.read_text(errors="replace")
 
     vdf_data = lower_dict(vdf.loads(content))
 
@@ -625,12 +706,23 @@ def find_steam_compat_tool_app(steam_path, steam_apps, appid=None):
         # App ID was most likely not provided
         app_section = None
 
+    try:
+        manifest_app_compat_section = next(
+            entry for mapping_appid, entry in
+            steam_play_manifest["appinfo"]["extended"]["app_mappings"].items()
+            if int(mapping_appid) == appid
+        )
+    except StopIteration:
+        # App doesn't have a default compatibility tool mapping
+        manifest_app_compat_section = None
+
     # ToolMapping seems to be used in older Steam beta releases
     try:
         tool_mapping = (
             vdf_data["installconfigstore"]["software"]["valve"]["steam"]
                     ["toolmapping"]
         )
+        logger.debug("Found ToolMapping entry")
     except KeyError:
         tool_mapping = {}
 
@@ -641,6 +733,7 @@ def find_steam_compat_tool_app(steam_path, steam_apps, appid=None):
             vdf_data["installconfigstore"]["software"]["valve"]["steam"]
                     ["compattoolmapping"]
         )
+        logger.debug("Found CompatToolMapping entry")
     except KeyError:
         compat_tool_mapping = {}
 
@@ -678,7 +771,19 @@ def find_steam_compat_tool_app(steam_path, steam_apps, appid=None):
             )
             potential_names.append(recommended_runtime)
 
-    # Global user settings have the 3rd highest priority
+    # Game specific default compatibility tool mapping in Steam Play 2.0
+    # manifest has the 3rd highest priority
+    if manifest_app_compat_section:
+        if "tool" in manifest_app_compat_section:
+            tool = manifest_app_compat_section["tool"]
+            logger.info(
+                "App has default compatibility tool mapping in the Steam Play "
+                "manifest: %s",
+                tool
+            )
+            potential_names.append(tool)
+
+    # Global user settings have the 4th highest priority
     if compat_tool_mapping.get("0", {}).get("name"):
         tool_name = compat_tool_mapping["0"]["name"]
         logger.info(
@@ -688,7 +793,7 @@ def find_steam_compat_tool_app(steam_path, steam_apps, appid=None):
         )
         potential_names.append(tool_name)
 
-    # Legacy user settings (ToolMapping) have the 4th highest priority
+    # Legacy user settings (ToolMapping) have the 5th highest priority
     if tool_mapping.get(str(appid), {}).get("name", {}):
         tool_name = tool_mapping[str(appid)]["name"]
         logger.info(
@@ -779,6 +884,10 @@ def find_appid_proton_prefix(appid, steam_lib_paths):
             if prefix_path.is_dir():
                 candidates.append(prefix_path)
 
+    logger.debug(
+        "Found compatdata directories for app %s: %s", appid, candidates
+    )
+
     if len(candidates) > 1:
         # If we have more than one possible prefix path, use the one
         # with the most recent modification date
@@ -883,6 +992,12 @@ def get_steam_lib_paths(steam_path):
             if is_library_folder_xdg_steam and is_flatpak_steam:
                 path = flatpak_steam_path
 
+            logger.debug(
+                "Found Steam library folder %s. Is Flatpak path: %s, "
+                "Is XDG Steam path: %s.",
+                path, is_flatpak_steam, is_library_folder_xdg_steam
+            )
+
             library_folders.append(path)
 
         logger.info(
@@ -934,6 +1049,10 @@ def get_compat_tool_dirs(steam_root):
     ]
     extra_ct_paths_env = os.getenv("STEAM_EXTRA_COMPAT_TOOLS_PATHS")
     if extra_ct_paths_env:
+        logger.debug(
+            "Including extra compat tool paths provided via env var: %s",
+            extra_ct_paths_env
+        )
         paths += [Path(path) for path in extra_ct_paths_env.split(os.pathsep)]
     paths += [steam_root / "compatibilitytools.d"]
 
@@ -956,6 +1075,10 @@ def get_custom_compat_tool_installations_in_dir(compat_tool_dir):
     for vdf_path in comptool_files:
         content = vdf_path.read_text()
 
+        logger.debug(
+            "Parsing custom compatibility tool manifest at %s", vdf_path
+        )
+
         try:
             vdf_data = vdf.loads(content)
         except SyntaxError:
@@ -966,24 +1089,40 @@ def get_custom_compat_tool_installations_in_dir(compat_tool_dir):
             )
             continue
 
+        if not vdf_data:
+            logger.warning(
+                "Compatibility tool declaration at %s is empty. You may need "
+                "to reinstall the application.",
+                vdf_path
+            )
+            continue
+
         # Traverse to 'compatibilitytools/compat_tools' in a case-insensitive
         # way. This is done because we can't turn all keys recursively to
         # lowercase from the get-go; the app name is stored as a key.
-        compat_tools = {k.lower(): v for k, v in vdf_data.items()}
-        compat_tools = compat_tools["compatibilitytools"]
-        compat_tools = {
-            k.lower(): v for k, v in compat_tools.items()
-        }
-        compat_tools = compat_tools["compat_tools"]
-        internal_name = list(compat_tools.keys())[0]
-        tool_info = compat_tools[internal_name]
+        try:
+            compat_tools = {k.lower(): v for k, v in vdf_data.items()}
+            compat_tools = compat_tools["compatibilitytools"]
+            compat_tools = {
+                k.lower(): v for k, v in compat_tools.items()
+            }
+            compat_tools = compat_tools["compat_tools"]
+            internal_name = list(compat_tools.keys())[0]
+            tool_info = compat_tools[internal_name]
 
-        # We can now convert the remainder into lowercase
-        tool_info = lower_dict(tool_info)
+            # We can now convert the remainder into lowercase
+            tool_info = lower_dict(tool_info)
 
-        install_path_name = tool_info["install_path"]
-        from_oslist = tool_info["from_oslist"]
-        to_oslist = tool_info["to_oslist"]
+            install_path_name = tool_info["install_path"]
+            from_oslist = tool_info["from_oslist"]
+            to_oslist = tool_info["to_oslist"]
+        except KeyError:
+            logger.warning(
+                "Compatibility tool declaration at %s is incomplete. You may "
+                "need to reinstall the application",
+                vdf_path
+            )
+            continue
 
         if from_oslist != "windows" or to_oslist != "linux":
             continue
@@ -1009,6 +1148,11 @@ def get_custom_compat_tool_installations_in_dir(compat_tool_dir):
             )
             continue
 
+        logger.debug(
+            "Found custom compatibility tool %s at %s",
+            internal_name, install_path
+        )
+
         custom_tool_apps.append(
             SteamApp(
                 name=internal_name, install_path=install_path,
@@ -1069,6 +1213,8 @@ def find_current_steamid3(steam_path):
         for user_id, user_data in user_datas
     ]
 
+    logger.debug("Found Steam user entries: %s", users)
+
     # Return the user with the highest timestamp, as that's likely to be the
     # currently logged-in user
     if users:
@@ -1175,11 +1321,23 @@ def get_custom_windows_shortcuts(steam_path, steam_lib_paths):
             )
             continue
 
+        icon_path = None
+
+        if shortcut_data.get("icon", None):
+            icon_path = Path(shortcut_data["icon"])
+
+        logger.debug(
+            "Creating SteamApp from non-Steam shortcut in %s", install_path
+        )
+
         steam_apps.append(
             SteamApp(
                 appid=appid,
                 name=f"Non-Steam shortcut: {shortcut_data['appname']}",
-                prefix_path=prefix_path, install_path=install_path
+                prefix_path=prefix_path,
+                install_path=install_path,
+                icon_path=icon_path
+
             )
         )
 
@@ -1251,7 +1409,8 @@ def get_steam_apps(steam_root, steam_path, steam_lib_paths):
         for manifest_path in appmanifest_paths:
             try:
                 steam_app = SteamApp.from_appmanifest(
-                    manifest_path, steam_lib_paths=steam_lib_paths
+                    manifest_path, steam_lib_paths=steam_lib_paths,
+                    steam_path=steam_path
                 )
             except PermissionError as exc:
                 logger.warning(
