@@ -6,7 +6,7 @@ import shutil
 import stat
 import tempfile
 from pathlib import Path
-from subprocess import DEVNULL, PIPE, Popen, check_output, run
+from subprocess import DEVNULL, PIPE, Popen, TimeoutExpired, check_output, run
 
 import pkg_resources
 
@@ -15,14 +15,19 @@ __all__ = (
     "is_steam_deck", "get_legacy_runtime_library_paths",
     "get_host_library_paths", "RUNTIME_ROOT_GLOB_PATTERNS",
     "get_runtime_library_paths", "WINE_SCRIPT_TEMPLATE",
-    "create_wine_bin_dir", "run_command"
+    "get_cache_dir", "create_wine_bin_dir", "run_command"
 )
 
 logger = logging.getLogger("protontricks")
 
 SUPPORTED_STEAM_RUNTIMES = [
+    # Old names
     "Steam Linux Runtime - Soldier",
-    "Steam Linux Runtime - Sniper"
+    "Steam Linux Runtime - Sniper",
+
+    # New names
+    "Steam Linux Runtime 2.0 (soldier)",
+    "Steam Linux Runtime 3.0 (sniper)"
 ]
 
 OS_RELEASE_PATHS = [
@@ -171,6 +176,20 @@ BWRAP_LAUNCHER_SH_SCRIPT = Path(
 ).read_text(encoding="utf-8")
 
 
+def get_cache_dir():
+    """
+    Get Protontricks' cache directory, creating it first if it does not
+    exist
+    """
+    xdg_cache_dir = os.environ.get(
+        "XDG_CACHE_HOME", os.path.expanduser("~/.cache")
+    )
+    base_path = Path(xdg_cache_dir) / "protontricks"
+    os.makedirs(str(base_path), exist_ok=True)
+
+    return base_path
+
+
 def create_wine_bin_dir(proton_app, use_bwrap=True):
     """
     Create a directory with "proxy" executables that load shared libraries
@@ -180,10 +199,7 @@ def create_wine_bin_dir(proton_app, use_bwrap=True):
     binaries = list((proton_app.proton_dist_path / "bin").iterdir())
 
     # Create the base directory containing files for every Proton installation
-    xdg_cache_dir = os.environ.get(
-        "XDG_CACHE_HOME", os.path.expanduser("~/.cache")
-    )
-    base_path = Path(xdg_cache_dir) / "protontricks" / "proton"
+    base_path = get_cache_dir() / "proton"
     os.makedirs(str(base_path), exist_ok=True)
 
     # Create a directory to hold the new executables for the specific
@@ -474,13 +490,39 @@ def run_command(
                 "Starting bwrap launcher process: %s",
                 str(wine_bin_dir / "bwrap-launcher")
             )
+
+            # TODO: Waiting for launcher to start can be simplified once
+            # ValveSoftware/steam-runtime#593 has been fixed and stdout can
+            # be used instead.
+            launcher_read_fd, launcher_write_fd = os.pipe2(os.O_CLOEXEC)
             launcher_process = _start_process(
-                str(wine_bin_dir / "bwrap-launcher"),
+                [str(wine_bin_dir / "bwrap-launcher"), str(launcher_write_fd)],
                 wait=False,
-                env=wine_environ,
-                stdout=DEVNULL,
+                pass_fds=[launcher_write_fd],
+                env=wine_environ
             )
 
+            # The Steam Runtime launcher service will write to the given
+            # file descriptor and then close it to indicate the launcher is
+            # ready or about to exit (i.e. due to wrong CLI parameters).
+            os.close(launcher_write_fd)
+            with open(launcher_read_fd, "rb") as reader:
+                reader.read()
+
+            # Check if the launcher actually started up and is still running.
+            try:
+                launcher_process.wait(timeout=0.1)
+                # Launcher process crashed, bail out
+                raise RuntimeError(
+                    f"bwrap launcher crashed, "
+                    f"returncode: {launcher_process.returncode}"
+                )
+            except TimeoutExpired:
+                # Launcher is running as expected
+                pass
+
+            logger.info("bwrap launcher started")
+
         if start_wineserver:
             logger.info(
                 "Starting wineserver keepalive process: %s",
@@ -505,14 +547,14 @@ def run_command(
     finally:
         shutil.rmtree(str(temp_dir), ignore_errors=True)
 
-        if start_wineserver:
+        if keepalive_process:
             logger.info(
                 "Terminating wineserver keepalive process %d",
                 keepalive_process.pid
             )
             keepalive_process.terminate()
 
-        if use_bwrap:
+        if launcher_process:
             logger.info(
                 "Terminating launcher process %d",
                 launcher_process.pid
