@@ -1,4 +1,5 @@
 import os
+import sys
 import shutil
 from pathlib import Path
 
@@ -334,15 +335,6 @@ class TestCLIRun:
 
         assert "Steam app with the given app ID could not be found" in result
 
-    def test_run_no_command(self, cli):
-        """
-        Run only the 'protontricks' command.
-        """
-        result = cli([])
-
-        # Help will be printed if no specific command is given
-        assert result.startswith("usage: ")
-
     @pytest.mark.usefixtures("default_proton")
     def test_run_returncode_passed(self, cli, steam_app_factory):
         """
@@ -780,6 +772,23 @@ class TestCLIGUI:
 
         assert "Proton installation is incomplete" in result
 
+    @pytest.mark.usefixtures("default_proton", "gui_provider")
+    def test_run_no_args(
+            self, cli, steam_app_factory, command_mock, gui_provider,
+            monkeypatch):
+        """
+        Run only the 'protontricks' command. This will default to GUI.
+        """
+        # Monkeypatch 'sys.argv', as that seems to be the only way to determine
+        # whether no arguments were provided
+        monkeypatch.setattr(sys, "argv", ["protontricks"])
+        steam_app_factory(name="Fake game", appid=10)
+
+        result = cli([], expect_returncode=1)
+
+        # Help will be printed if no specific command is given
+        assert "No game was selected" in result
+
 
 class TestCLICommand:
     def test_run_command(
@@ -797,8 +806,7 @@ class TestCLICommand:
 
         # The command is just 'bash'
         assert command.args == "bash"
-
-        assert command.cwd == str(steam_app.install_path)
+        assert command.cwd is None
         assert command.shell is True
 
         # Correct environment vars were set
@@ -817,6 +825,22 @@ class TestCLICommand:
             str(proton_install_path / "dist" / "lib" / "wine")
         )
 
+    @pytest.mark.usefixtures("default_proton")
+    def test_run_command_cwd_app(self, cli, steam_app_factory, command_mock):
+        """
+        Run a shell command for a given game using `--cwd-app` flag and
+        ensure the working directory was set to the game's installation
+        directory
+        """
+        steam_app = steam_app_factory(name="Fake game", appid=10)
+
+        cli(["--cwd-app", "-c", "bash", "10"])
+
+        command = command_mock.commands[-1]
+
+        assert command.args == "bash"
+        assert command.cwd == str(steam_app.install_path)
+
 
 class TestCLISearch:
     def test_search_case_insensitive(self, cli, steam_app_factory):
